@@ -4,6 +4,8 @@ import (
 	"testing"
 
 	"github.com/evanw/esbuild/internal/config"
+	"github.com/evanw/esbuild/internal/js_ast"
+	"github.com/evanw/esbuild/internal/logger"
 )
 
 var default_suite = suite{
@@ -227,7 +229,7 @@ func TestExportFormsIIFE(t *testing.T) {
 		options: config.Options{
 			Mode:          config.ModeBundle,
 			OutputFormat:  config.FormatIIFE,
-			ModuleName:    "moduleName",
+			GlobalName:    []string{"globalName"},
 			AbsOutputFile: "/out.js",
 		},
 	})
@@ -676,23 +678,49 @@ func TestImportMissingCommonJS(t *testing.T) {
 func TestImportMissingNeitherES6NorCommonJS(t *testing.T) {
 	default_suite.expectBundled(t, bundled{
 		files: map[string]string{
-			"/entry.js": `
+			"/named.js": `
 				import fn, {x as a, y as b} from './foo'
-				import * as ns from './foo'
 				console.log(fn(a, b))
 			`,
+			"/star.js": `
+				import * as ns from './foo'
+				console.log(ns.default(ns.x, ns.y))
+			`,
+			"/star-capture.js": `
+				import * as ns from './foo'
+				console.log(ns)
+			`,
+			"/bare.js": `
+				import './foo'
+			`,
+			"/require.js": `
+				console.log(require('./foo'))
+			`,
+			"/import.js": `
+				console.log(import('./foo'))
+			`,
 			"/foo.js": `
 				console.log('no exports here')
 			`,
 		},
-		entryPaths: []string{"/entry.js"},
-		options: config.Options{
-			Mode:          config.ModeBundle,
-			AbsOutputFile: "/out.js",
+		entryPaths: []string{
+			"/named.js",
+			"/star.js",
+			"/star-capture.js",
+			"/bare.js",
+			"/require.js",
+			"/import.js",
 		},
-		expectedCompileLog: `/entry.js: warning: Import "default" will always be undefined
-/entry.js: warning: Import "x" will always be undefined
-/entry.js: warning: Import "y" will always be undefined
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+		},
+		expectedCompileLog: `/named.js: warning: Import "default" will always be undefined
+/named.js: warning: Import "x" will always be undefined
+/named.js: warning: Import "y" will always be undefined
+/star.js: warning: Import "default" will always be undefined
+/star.js: warning: Import "x" will always be undefined
+/star.js: warning: Import "y" will always be undefined
 `,
 	})
 }
@@ -787,6 +815,15 @@ func TestRequireAndDynamicImportInvalidTemplate(t *testing.T) {
 				require(` + "`./${b}`" + `)
 				import(tag` + "`./b`" + `)
 				import(` + "`./${b}`" + `)
+
+				// Try/catch should silence this warning for require()
+				try {
+					require(tag` + "`./b`" + `)
+					require(` + "`./${b}`" + `)
+					import(tag` + "`./b`" + `)
+					import(` + "`./${b}`" + `)
+				} catch {
+				}
 			`,
 		},
 		entryPaths: []string{"/entry.js"},
@@ -798,6 +835,157 @@ func TestRequireAndDynamicImportInvalidTemplate(t *testing.T) {
 /entry.js: warning: This call to "require" will not be bundled because the argument is not a string literal
 /entry.js: warning: This dynamic import will not be bundled because the argument is not a string literal
 /entry.js: warning: This dynamic import will not be bundled because the argument is not a string literal
+/entry.js: warning: This dynamic import will not be bundled because the argument is not a string literal
+/entry.js: warning: This dynamic import will not be bundled because the argument is not a string literal
+`,
+	})
+}
+
+func TestDynamicImportWithExpressionCJS(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import('foo')
+				import(foo())
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:          config.ModeConvertFormat,
+			OutputFormat:  config.FormatCommonJS,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestDynamicImportWithExpressionCJSAndES5(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import('foo')
+				import(foo())
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:                  config.ModeConvertFormat,
+			OutputFormat:          config.FormatCommonJS,
+			UnsupportedJSFeatures: es(5),
+			AbsOutputFile:         "/out.js",
+		},
+	})
+}
+
+func TestMinifiedDynamicImportWithExpressionCJS(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import('foo')
+				import(foo())
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:             config.ModeConvertFormat,
+			OutputFormat:     config.FormatCommonJS,
+			AbsOutputFile:    "/out.js",
+			RemoveWhitespace: true,
+		},
+	})
+}
+
+func TestMinifiedDynamicImportWithExpressionCJSAndES5(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import('foo')
+				import(foo())
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:                  config.ModeConvertFormat,
+			OutputFormat:          config.FormatCommonJS,
+			UnsupportedJSFeatures: es(5),
+			AbsOutputFile:         "/out.js",
+			RemoveWhitespace:      true,
+		},
+	})
+}
+
+func TestConditionalRequireResolve(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				require.resolve(x ? 'a' : y ? 'b' : 'c')
+				require.resolve(x ? y ? 'a' : 'b' : c)
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"a": true,
+					"b": true,
+					"c": true,
+				},
+			},
+		},
+	})
+}
+
+func TestConditionalRequire(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				require(x ? 'a' : y ? './b' : 'c')
+				require(x ? y ? 'a' : './b' : c)
+			`,
+			"/b.js": `
+				exports.foo = 213
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"a": true,
+					"c": true,
+				},
+			},
+		},
+		expectedScanLog: `/a.js: warning: This call to "require" will not be bundled because the argument is not a string literal
+`,
+	})
+}
+
+func TestConditionalImport(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import(x ? 'a' : y ? './b' : 'c')
+				import(x ? y ? 'a' : './b' : c)
+			`,
+			"/b.js": `
+				exports.foo = 213
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"a": true,
+					"c": true,
+				},
+			},
+		},
+		expectedScanLog: `/a.js: warning: This dynamic import will not be bundled because the argument is not a string literal
 `,
 	})
 }
@@ -808,6 +996,13 @@ func TestRequireBadArgumentCount(t *testing.T) {
 			"/entry.js": `
 				require()
 				require("a", "b")
+
+				// Try/catch should silence this warning
+				try {
+					require()
+					require("a", "b")
+				} catch {
+				}
 			`,
 		},
 		entryPaths: []string{"/entry.js"},
@@ -872,7 +1067,7 @@ func TestRequireBadExtension(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `/entry.js: error: File extension not supported: /test
+		expectedScanLog: `/entry.js: error: File could not be loaded: /test
 `,
 	})
 }
@@ -1947,7 +2142,7 @@ func TestExternalModuleExclusionRelativePath(t *testing.T) {
 }
 
 func TestAutoExternal(t *testing.T) {
-	css_suite.expectBundled(t, bundled{
+	default_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
 				// These URLs should be external automatically
@@ -1965,6 +2160,41 @@ func TestAutoExternal(t *testing.T) {
 	})
 }
 
+func TestExternalWithWildcard(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				// Should match
+				import "/assets/images/test.jpg";
+				import "/dir/x/file.gif";
+				import "/dir//file.gif";
+				import "./file.png";
+
+				// Should not match
+				import "/sassets/images/test.jpg";
+				import "/dir/file.gif";
+				import "./file.ping";
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+			ExternalModules: config.ExternalModules{
+				Patterns: []config.WildcardPattern{
+					{Prefix: "/assets/"},
+					{Suffix: ".png"},
+					{Prefix: "/dir/", Suffix: "/file.gif"},
+				},
+			},
+		},
+		expectedScanLog: `/entry.js: error: Could not read from file: /sassets/images/test.jpg
+/entry.js: error: Could not read from file: /dir/file.gif
+/entry.js: error: Could not resolve "./file.ping"
+`,
+	})
+}
+
 // This test case makes sure many entry points don't cause a crash
 func TestManyEntryPoints(t *testing.T) {
 	default_suite.expectBundled(t, bundled{
@@ -2559,9 +2789,9 @@ func TestOutputExtensionRemappingFile(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			Mode:             config.ModeBundle,
-			OutputExtensions: map[string]string{".js": ".notjs"},
-			AbsOutputFile:    "/out.js",
+			Mode:              config.ModeBundle,
+			OutputExtensionJS: ".notjs",
+			AbsOutputFile:     "/out.js",
 		},
 	})
 }
@@ -2575,9 +2805,9 @@ func TestOutputExtensionRemappingDir(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			Mode:             config.ModeBundle,
-			OutputExtensions: map[string]string{".js": ".notjs"},
-			AbsOutputDir:     "/out",
+			Mode:              config.ModeBundle,
+			OutputExtensionJS: ".notjs",
+			AbsOutputDir:      "/out",
 		},
 	})
 }
@@ -2688,6 +2918,11 @@ func TestAssignToImport(t *testing.T) {
 				import "./bad8.js"
 				import "./bad9.js"
 				import "./bad10.js"
+				import "./bad11.js"
+				import "./bad12.js"
+				import "./bad13.js"
+				import "./bad14.js"
+				import "./bad15.js"
 
 				import "./good0.js"
 				import "./good1.js"
@@ -2708,6 +2943,11 @@ func TestAssignToImport(t *testing.T) {
 			"/bad8.js":  `import * as x from "foo"; x[y] = 1`,
 			"/bad9.js":  `import * as x from "foo"; x['y'] = 1`,
 			"/bad10.js": `import * as x from "foo"; x['y z'] = 1`,
+			"/bad11.js": `import x from "foo"; delete x`,
+			"/bad12.js": `import {x} from "foo"; delete x`,
+			"/bad13.js": `import * as x from "foo"; delete x.y`,
+			"/bad14.js": `import * as x from "foo"; delete x['y']`,
+			"/bad15.js": `import * as x from "foo"; delete x[y]`,
 
 			"/good0.js": `import x from "foo"; ({y = x} = 1)`,
 			"/good1.js": `import x from "foo"; ({[x]: y} = 1)`,
@@ -2724,6 +2964,11 @@ func TestAssignToImport(t *testing.T) {
 		expectedScanLog: `/bad0.js: error: Cannot assign to import "x"
 /bad1.js: error: Cannot assign to import "x"
 /bad10.js: error: Cannot assign to import "y z"
+/bad11.js: error: Cannot assign to import "x"
+/bad12.js: error: Cannot assign to import "x"
+/bad13.js: error: Cannot assign to import "y"
+/bad14.js: error: Cannot assign to import "y"
+/bad15.js: error: Cannot assign to property on import "x"
 /bad2.js: error: Cannot assign to import "x"
 /bad3.js: error: Cannot assign to import "x"
 /bad4.js: error: Cannot assign to import "x"
@@ -2834,7 +3079,7 @@ func TestWarningsInsideNodeModules(t *testing.T) {
 }
 
 func TestRequireResolve(t *testing.T) {
-	splitting_suite.expectBundled(t, bundled{
+	default_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
 				console.log(require.resolve)
@@ -2855,6 +3100,11 @@ func TestRequireResolve(t *testing.T) {
 				if (false) {
 					console.log(require.resolve('dead-code'))
 				}
+				console.log(false ? require.resolve('dead-if') : 0)
+				console.log(true ? 0 : require.resolve('dead-if'))
+				console.log(false && require.resolve('dead-and'))
+				console.log(true || require.resolve('dead-or'))
+				console.log(true ?? require.resolve('dead-nullish'))
 			`,
 			"/present-file.js": ``,
 		},
@@ -2875,7 +3125,6 @@ func TestRequireResolve(t *testing.T) {
 		expectedScanLog: `/entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
 /entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
 /entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
-/entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
 /entry.js: warning: "./present-file" should be marked as external for use with "require.resolve"
 /entry.js: warning: "./missing-file" should be marked as external for use with "require.resolve"
 /entry.js: warning: "missing-pkg" should be marked as external for use with "require.resolve"
@@ -2883,3 +3132,592 @@ func TestRequireResolve(t *testing.T) {
 `,
 	})
 }
+
+func TestInjectMissing(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": ``,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			InjectAbsPaths: []string{
+				"/inject.js",
+			},
+		},
+		expectedScanLog: `error: Could not read from file: /inject.js
+`,
+	})
+}
+
+func TestInjectDuplicate(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js":  ``,
+			"/inject.js": ``,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			InjectAbsPaths: []string{
+				"/inject.js",
+				"/inject.js",
+			},
+		},
+		expectedScanLog: `error: Duplicate injected file "/inject.js"
+`,
+	})
+}
+
+func TestInject(t *testing.T) {
+	defines := config.ProcessDefines(map[string]config.DefineData{
+		"chain.prop": {
+			DefineFunc: func(loc logger.Loc, findSymbol config.FindSymbol) js_ast.E {
+				return &js_ast.EIdentifier{Ref: findSymbol(loc, "replace")}
+			},
+		},
+	})
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				let sideEffects = console.log('this should be renamed')
+				let collide = 123
+				console.log(obj.prop)
+				console.log(chain.prop.test)
+				console.log(collide)
+				console.log(re_export)
+			`,
+			"/inject.js": `
+				export let obj = {}
+				export let sideEffects = console.log('side effects')
+				export let noSideEffects = /* @__PURE__ */ console.log('side effects')
+			`,
+			"/node_modules/unused/index.js": `
+				console.log('This is unused but still has side effects')
+			`,
+			"/node_modules/sideEffects-false/index.js": `
+				console.log('This is unused and has no side effects')
+			`,
+			"/node_modules/sideEffects-false/package.json": `{
+				"sideEffects": false
+			}`,
+			"/replacement.js": `
+				export let replace = {
+					test() {}
+				}
+			`,
+			"/collision.js": `
+				export let collide = 123
+			`,
+			"/re-export.js": `
+				export {re_export} from 'external-pkg'
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			Defines:       &defines,
+			OutputFormat:  config.FormatCommonJS,
+			InjectAbsPaths: []string{
+				"/inject.js",
+				"/node_modules/unused/index.js",
+				"/node_modules/sideEffects-false/index.js",
+				"/replacement.js",
+				"/collision.js",
+				"/re-export.js",
+			},
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"external-pkg": true,
+				},
+			},
+		},
+	})
+}
+
+func TestInjectNoBundle(t *testing.T) {
+	defines := config.ProcessDefines(map[string]config.DefineData{
+		"chain.prop": {
+			DefineFunc: func(loc logger.Loc, findSymbol config.FindSymbol) js_ast.E {
+				return &js_ast.EIdentifier{Ref: findSymbol(loc, "replace")}
+			},
+		},
+	})
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				let sideEffects = console.log('this should be renamed')
+				let collide = 123
+				console.log(obj.prop)
+				console.log(chain.prop.test)
+				console.log(collide)
+				console.log(re_export)
+			`,
+			"/inject.js": `
+				export let obj = {}
+				export let sideEffects = console.log('side effects')
+				export let noSideEffects = /* @__PURE__ */ console.log('side effects')
+			`,
+			"/node_modules/unused/index.js": `
+				console.log('This is unused but still has side effects')
+			`,
+			"/node_modules/sideEffects-false/index.js": `
+				console.log('This is unused and has no side effects')
+			`,
+			"/node_modules/sideEffects-false/package.json": `{
+				"sideEffects": false
+			}`,
+			"/replacement.js": `
+				export let replace = {
+					test() {}
+				}
+			`,
+			"/collision.js": `
+				export let collide = 123
+			`,
+			"/re-export.js": `
+				export {re_export} from 'external-pkg'
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModePassThrough,
+			AbsOutputFile: "/out.js",
+			Defines:       &defines,
+			InjectAbsPaths: []string{
+				"/inject.js",
+				"/node_modules/unused/index.js",
+				"/node_modules/sideEffects-false/index.js",
+				"/replacement.js",
+				"/collision.js",
+				"/re-export.js",
+			},
+		},
+	})
+}
+
+func TestInjectJSX(t *testing.T) {
+	defines := config.ProcessDefines(map[string]config.DefineData{
+		"React.createElement": {
+			DefineFunc: func(loc logger.Loc, findSymbol config.FindSymbol) js_ast.E {
+				return &js_ast.EIdentifier{Ref: findSymbol(loc, "el")}
+			},
+		},
+	})
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.jsx": `
+				console.log(<div/>)
+			`,
+			"/inject.js": `
+				export function el() {}
+			`,
+		},
+		entryPaths: []string{"/entry.jsx"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			Defines:       &defines,
+			InjectAbsPaths: []string{
+				"/inject.js",
+			},
+		},
+	})
+}
+
+func TestInjectImportTS(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				console.log('here')
+			`,
+			"/inject.js": `
+				// Unused imports are automatically removed in TypeScript files (this
+				// is a mis-feature of the TypeScript language). However, injected
+				// imports are an esbuild feature so we get to decide what the
+				// semantics are. We do not want injected imports to disappear unless
+				// they have been explicitly marked as having no side effects.
+				console.log('must be present')
+			`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeConvertFormat,
+			OutputFormat:  config.FormatESModule,
+			AbsOutputFile: "/out.js",
+			InjectAbsPaths: []string{
+				"/inject.js",
+			},
+		},
+	})
+}
+
+func TestInjectImportOrder(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				import 'third'
+				console.log('third')
+			`,
+			"/inject-1.js": `
+				import 'first'
+				console.log('first')
+			`,
+			"/inject-2.js": `
+				import 'second'
+				console.log('second')
+			`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			InjectAbsPaths: []string{
+				"/inject-1.js",
+				"/inject-2.js",
+			},
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"first":  true,
+					"second": true,
+					"third":  true,
+				},
+			},
+		},
+	})
+}
+
+func TestOutbase(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a/b/c.js": `
+				console.log('c')
+			`,
+			"/a/b/d.js": `
+				console.log('d')
+			`,
+		},
+		entryPaths: []string{"/a/b/c.js", "/a/b/d.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputDir:  "/out",
+			AbsOutputBase: "/",
+		},
+	})
+}
+
+func TestAvoidTDZ(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				class Foo {
+					static foo = new Foo
+				}
+				let foo = Foo.foo
+				console.log(foo)
+				export class Bar {}
+				export let bar = 123
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestAvoidTDZNoBundle(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				class Foo {
+					static foo = new Foo
+				}
+				let foo = Foo.foo
+				console.log(foo)
+				export class Bar {}
+				export let bar = 123
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModePassThrough,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestProcessEnvNodeEnvWarning(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				console.log(
+					process.env.NODE_ENV,
+					process.env.NODE_ENV,
+				)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+		expectedScanLog: `/entry.js: warning: Define "process.env.NODE_ENV" when bundling for the browser
+`,
+	})
+}
+
+func TestProcessEnvNodeEnvWarningNode(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				console.log(process.env.NODE_ENV)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			Platform:      config.PlatformNode,
+		},
+	})
+}
+
+func TestProcessEnvNodeEnvWarningDefine(t *testing.T) {
+	defines := config.ProcessDefines(map[string]config.DefineData{
+		"process.env.NODE_ENV": {
+			DefineFunc: func(loc logger.Loc, findSymbol config.FindSymbol) js_ast.E {
+				return &js_ast.ENull{}
+			},
+		},
+	})
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				console.log(process.env.NODE_ENV)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			Defines:       &defines,
+		},
+	})
+}
+
+func TestProcessEnvNodeEnvWarningNoBundle(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				console.log(process.env.NODE_ENV)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModePassThrough,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestKeepNamesTreeShaking(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function fnStmtRemove() {}
+				function fnStmtKeep() {}
+				fnStmtKeep()
+
+				let fnExprRemove = function remove() {}
+				let fnExprKeep = function keep() {}
+				fnExprKeep()
+
+				class clsStmtRemove {}
+				class clsStmtKeep {}
+				new clsStmtKeep()
+
+				let clsExprRemove = class remove {}
+				let clsExprKeep = class keep {}
+				new clsExprKeep()
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			KeepNames:     true,
+			MangleSyntax:  true,
+		},
+	})
+}
+
+func TestCharFreqIgnoreComments(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				export default function(one, two, three, four) {
+					return 'the argument names must be the same'
+				}
+			`,
+			"/b.js": `
+				export default function(one, two, three, four) {
+					return 'the argument names must be the same'
+				}
+
+				// Some comment text to change the character frequency histogram:
+				// ________________________________________________________________________________
+				// FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
+				// AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
+				// IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
+				// LLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL
+			`,
+		},
+		entryPaths: []string{"/a.js", "/b.js"},
+		options: config.Options{
+			Mode:              config.ModeBundle,
+			AbsOutputDir:      "/out",
+			MinifyIdentifiers: true,
+		},
+	})
+}
+
+func TestImportRelativeAsPackage(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import 'some/other/file'
+			`,
+			"/Users/user/project/src/some/other/file.js": `
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+		expectedScanLog: `/Users/user/project/src/entry.js: error: Could not resolve "some/other/file" ` +
+			`(use "./some/other/file" to import "/Users/user/project/src/some/other/file.js")
+`,
+	})
+}
+
+func TestForbidConstAssignWhenBundling(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				const x = 1
+				x = 2
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+		expectedScanLog: `/entry.js: error: Cannot assign to "x" because it is a constant
+/entry.js: note: "x" was declared a constant here
+`,
+	})
+}
+
+func TestConstWithLet(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				const a = 1; console.log(a)
+				if (true) { const b = 2; console.log(b) }
+				for (const c = x;;) console.log(c)
+				for (const d in x) console.log(d)
+				for (const e of x) console.log(e)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MangleSyntax:  true,
+		},
+	})
+}
+
+func TestConstWithLetNoBundle(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				const a = 1; console.log(a)
+				if (true) { const b = 2; console.log(b) }
+				for (const c = x;;) console.log(c)
+				for (const d in x) console.log(d)
+				for (const e of x) console.log(e)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModePassThrough,
+			AbsOutputFile: "/out.js",
+			MangleSyntax:  true,
+		},
+	})
+}
+
+func TestConstWithLetNoMangle(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				const a = 1; console.log(a)
+				if (true) { const b = 2; console.log(b) }
+				for (const c = x;;) console.log(c)
+				for (const d in x) console.log(d)
+				for (const e of x) console.log(e)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestRequireMainCommonJS(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				console.log('is main:', require.main === module)
+				console.log(require('./is-main'))
+			`,
+			"/is-main.js": `
+				module.exports = require.main === module
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			OutputFormat:  config.FormatCommonJS,
+		},
+	})
+}
+
+func TestRequireMainIIFE(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				console.log('is main:', require.main === module)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			OutputFormat:  config.FormatIIFE,
+		},
+		expectedScanLog: `/entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
+`,
+	})
+}
