@@ -943,33 +943,6 @@ func TestLowerAsyncSuperES2016NoBundle(t *testing.T) {
 	})
 }
 
-func TestLowerClassFieldStrict2020NoBundle(t *testing.T) {
-	lower_suite.expectBundled(t, bundled{
-		files: map[string]string{
-			"/entry.js": `
-				class Foo {
-					#foo = 123
-					#bar
-					foo = 123
-					bar
-					static #s_foo = 123
-					static #s_bar
-					static s_foo = 123
-					static s_bar
-				}
-			`,
-		},
-		entryPaths: []string{"/entry.js"},
-		options: config.Options{
-			UnsupportedJSFeatures: es(2020),
-			Strict: config.StrictOptions{
-				ClassFields: true,
-			},
-			AbsOutputFile: "/out.js",
-		},
-	})
-}
-
 func TestLowerClassField2020NoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -994,32 +967,6 @@ func TestLowerClassField2020NoBundle(t *testing.T) {
 	})
 }
 
-func TestLowerClassFieldStrictNextNoBundle(t *testing.T) {
-	lower_suite.expectBundled(t, bundled{
-		files: map[string]string{
-			"/entry.js": `
-				class Foo {
-					#foo = 123
-					#bar
-					foo = 123
-					bar
-					static #s_foo = 123
-					static #s_bar
-					static s_foo = 123
-					static s_bar
-				}
-			`,
-		},
-		entryPaths: []string{"/entry.js"},
-		options: config.Options{
-			Strict: config.StrictOptions{
-				ClassFields: true,
-			},
-			AbsOutputFile: "/out.js",
-		},
-	})
-}
-
 func TestLowerClassFieldNextNoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1043,33 +990,6 @@ func TestLowerClassFieldNextNoBundle(t *testing.T) {
 	})
 }
 
-func TestTSLowerClassFieldStrict2020NoBundle(t *testing.T) {
-	lower_suite.expectBundled(t, bundled{
-		files: map[string]string{
-			"/entry.ts": `
-				class Foo {
-					#foo = 123
-					#bar
-					foo = 123
-					bar
-					static #s_foo = 123
-					static #s_bar
-					static s_foo = 123
-					static s_bar
-				}
-			`,
-		},
-		entryPaths: []string{"/entry.ts"},
-		options: config.Options{
-			UnsupportedJSFeatures: es(2020),
-			Strict: config.StrictOptions{
-				ClassFields: true,
-			},
-			AbsOutputFile: "/out.js",
-		},
-	})
-}
-
 func TestTSLowerClassField2020NoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1094,32 +1014,6 @@ func TestTSLowerClassField2020NoBundle(t *testing.T) {
 	})
 }
 
-func TestTSLowerClassPrivateFieldStrictNextNoBundle(t *testing.T) {
-	lower_suite.expectBundled(t, bundled{
-		files: map[string]string{
-			"/entry.ts": `
-				class Foo {
-					#foo = 123
-					#bar
-					foo = 123
-					bar
-					static #s_foo = 123
-					static #s_bar
-					static s_foo = 123
-					static s_bar
-				}
-			`,
-		},
-		entryPaths: []string{"/entry.ts"},
-		options: config.Options{
-			Strict: config.StrictOptions{
-				ClassFields: true,
-			},
-			AbsOutputFile: "/out.js",
-		},
-	})
-}
-
 func TestTSLowerClassPrivateFieldNextNoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1185,6 +1079,48 @@ func TestLowerClassFieldStrictTsconfigJson2020(t *testing.T) {
 	})
 }
 
+func TestTSLowerClassFieldStrictTsconfigJson2020(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import loose from './loose'
+				import strict from './strict'
+				console.log(loose, strict)
+			`,
+			"/loose/index.ts": `
+				export default class {
+					foo
+				}
+			`,
+			"/loose/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"useDefineForClassFields": false
+					}
+				}
+			`,
+			"/strict/index.ts": `
+				export default class {
+					foo
+				}
+			`,
+			"/strict/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"useDefineForClassFields": true
+					}
+				}
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			UnsupportedJSFeatures: es(2020),
+			AbsOutputFile:         "/out.js",
+		},
+	})
+}
+
 func TestTSLowerObjectRest2017NoBundle(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1292,3 +1228,51 @@ func TestClassSuperThisIssue242NoBundle(t *testing.T) {
 		},
 	})
 }
+
+func TestLowerExportStarAsNameCollisionNoBundle(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				export * as ns from 'path'
+				let ns = 123
+				export {ns as sn}
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			UnsupportedJSFeatures: es(2019),
+			AbsOutputFile:         "/out.js",
+		},
+	})
+}
+
+func TestLowerExportStarAsNameCollision(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import * as test from './nested'
+				console.log(test.foo, test.oof)
+				export * as ns from 'path1'
+				let ns = 123
+				export {ns as sn}
+			`,
+			"/nested.js": `
+				export * as foo from 'path2'
+				let foo = 123
+				export {foo as oof}
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			UnsupportedJSFeatures: es(2019),
+			AbsOutputFile:         "/out.js",
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"path1": true,
+					"path2": true,
+				},
+			},
+		},
+	})
+}
