@@ -27,6 +27,7 @@ func TestTsConfigPaths(t *testing.T) {
 				import test3 from 'test3/foo'
 				import test4 from 'test4/foo'
 				import test5 from 'test5/foo'
+				import absolute from './absolute'
 				export default {
 					test0,
 					test1,
@@ -34,6 +35,7 @@ func TestTsConfigPaths(t *testing.T) {
 					test3,
 					test4,
 					test5,
+					absolute,
 				}
 			`,
 			"/Users/user/project/baseurl_dot/tsconfig.json": `
@@ -47,6 +49,7 @@ func TestTsConfigPaths(t *testing.T) {
 							"t*t3/foo": ["./test3-succ*s.ts"],
 							"test4/*": ["./test4-first/*", "./test4-second/*"],
 							"test5/*": ["./test5-first/*", "./test5-second/*"],
+							"/virtual/*": ["./actual/*"],
 						}
 					}
 				}
@@ -69,6 +72,12 @@ func TestTsConfigPaths(t *testing.T) {
 			"/Users/user/project/baseurl_dot/test5-second/foo.ts": `
 				export default 'test5-success'
 			`,
+			"/Users/user/project/baseurl_dot/absolute.ts": `
+				export {default} from '/virtual/test'
+			`,
+			"/Users/user/project/baseurl_dot/actual/test.ts": `
+				export default 'absolute-success'
+			`,
 
 			// Tests with "baseUrl": "nested"
 			"/Users/user/project/baseurl_nested/index.ts": `
@@ -78,6 +87,7 @@ func TestTsConfigPaths(t *testing.T) {
 				import test3 from 'test3/foo'
 				import test4 from 'test4/foo'
 				import test5 from 'test5/foo'
+				import absolute from './absolute'
 				export default {
 					test0,
 					test1,
@@ -85,6 +95,7 @@ func TestTsConfigPaths(t *testing.T) {
 					test3,
 					test4,
 					test5,
+					absolute,
 				}
 			`,
 			"/Users/user/project/baseurl_nested/tsconfig.json": `
@@ -98,6 +109,7 @@ func TestTsConfigPaths(t *testing.T) {
 							"t*t3/foo": ["./test3-succ*s.ts"],
 							"test4/*": ["./test4-first/*", "./test4-second/*"],
 							"test5/*": ["./test5-first/*", "./test5-second/*"],
+							"/virtual/*": ["./actual/*"],
 						}
 					}
 				}
@@ -120,6 +132,12 @@ func TestTsConfigPaths(t *testing.T) {
 			"/Users/user/project/baseurl_nested/nested/test5-second/foo.ts": `
 				export default 'test5-success'
 			`,
+			"/Users/user/project/baseurl_nested/absolute.ts": `
+				export {default} from '/virtual/test'
+			`,
+			"/Users/user/project/baseurl_nested/nested/actual/test.ts": `
+				export default 'absolute-success'
+			`,
 		},
 		entryPaths: []string{"/Users/user/project/entry.ts"},
 		options: config.Options{
@@ -129,6 +147,203 @@ func TestTsConfigPaths(t *testing.T) {
 	})
 }
 
+func TestTsConfigPathsNoBaseURL(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/entry.ts": `
+				import simple from './simple'
+				import extended from './extended'
+				console.log(simple, extended)
+			`,
+
+			// Tests with "baseUrl": "."
+			"/Users/user/project/simple/index.ts": `
+				import test0 from 'test0'
+				import test1 from 'test1/foo'
+				import test2 from 'test2/foo'
+				import test3 from 'test3/foo'
+				import test4 from 'test4/foo'
+				import test5 from 'test5/foo'
+				import absolute from './absolute'
+				export default {
+					test0,
+					test1,
+					test2,
+					test3,
+					test4,
+					test5,
+					absolute,
+				}
+			`,
+			"/Users/user/project/simple/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"paths": {
+							"test0": ["./test0-success.ts"],
+							"test1/*": ["./test1-success.ts"],
+							"test2/*": ["./test2-success/*"],
+							"t*t3/foo": ["./test3-succ*s.ts"],
+							"test4/*": ["./test4-first/*", "./test4-second/*"],
+							"test5/*": ["./test5-first/*", "./test5-second/*"],
+							"/virtual/*": ["./actual/*"],
+						}
+					}
+				}
+			`,
+			"/Users/user/project/simple/test0-success.ts": `
+				export default 'test0-success'
+			`,
+			"/Users/user/project/simple/test1-success.ts": `
+				export default 'test1-success'
+			`,
+			"/Users/user/project/simple/test2-success/foo.ts": `
+				export default 'test2-success'
+			`,
+			"/Users/user/project/simple/test3-success.ts": `
+				export default 'test3-success'
+			`,
+			"/Users/user/project/simple/test4-first/foo.ts": `
+				export default 'test4-success'
+			`,
+			"/Users/user/project/simple/test5-second/foo.ts": `
+				export default 'test5-success'
+			`,
+			"/Users/user/project/simple/absolute.ts": `
+				export {default} from '/virtual/test'
+			`,
+			"/Users/user/project/simple/actual/test.ts": `
+				export default 'absolute-success'
+			`,
+
+			// Tests with "baseUrl": "nested"
+			"/Users/user/project/extended/index.ts": `
+				import test0 from 'test0'
+				import test1 from 'test1/foo'
+				import test2 from 'test2/foo'
+				import test3 from 'test3/foo'
+				import test4 from 'test4/foo'
+				import test5 from 'test5/foo'
+				import absolute from './absolute'
+				export default {
+					test0,
+					test1,
+					test2,
+					test3,
+					test4,
+					test5,
+					absolute,
+				}
+			`,
+			"/Users/user/project/extended/tsconfig.json": `
+				{
+					"extends": "./nested/tsconfig.json"
+				}
+			`,
+			"/Users/user/project/extended/nested/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"paths": {
+							"test0": ["./test0-success.ts"],
+							"test1/*": ["./test1-success.ts"],
+							"test2/*": ["./test2-success/*"],
+							"t*t3/foo": ["./test3-succ*s.ts"],
+							"test4/*": ["./test4-first/*", "./test4-second/*"],
+							"test5/*": ["./test5-first/*", "./test5-second/*"],
+							"/virtual/*": ["./actual/*"],
+						}
+					}
+				}
+			`,
+			"/Users/user/project/extended/nested/test0-success.ts": `
+				export default 'test0-success'
+			`,
+			"/Users/user/project/extended/nested/test1-success.ts": `
+				export default 'test1-success'
+			`,
+			"/Users/user/project/extended/nested/test2-success/foo.ts": `
+				export default 'test2-success'
+			`,
+			"/Users/user/project/extended/nested/test3-success.ts": `
+				export default 'test3-success'
+			`,
+			"/Users/user/project/extended/nested/test4-first/foo.ts": `
+				export default 'test4-success'
+			`,
+			"/Users/user/project/extended/nested/test5-second/foo.ts": `
+				export default 'test5-success'
+			`,
+			"/Users/user/project/extended/absolute.ts": `
+				export {default} from '/virtual/test'
+			`,
+			"/Users/user/project/extended/nested/actual/test.ts": `
+				export default 'absolute-success'
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
+
+func TestTsConfigBadPathsNoBaseURL(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/entry.ts": `
+				import "should-not-be-imported"
+			`,
+			"/Users/user/project/should-not-be-imported.ts": `
+			`,
+			"/Users/user/project/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"paths": {
+							"test": [
+								".",
+								"..",
+								"./good",
+								".\\good",
+								"../good",
+								"..\\good",
+								"/good",
+								"\\good",
+								"c:/good",
+								"c:\\good",
+								"C:/good",
+								"C:\\good",
+
+								"bad",
+								"@bad/core",
+								".*/bad",
+								"..*/bad",
+								"c*:\\bad",
+								"c:*\\bad",
+								"http://bad"
+							]
+						}
+					}
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+		expectedScanLog: `/Users/user/project/entry.ts: error: Could not resolve "should-not-be-imported" ` +
+			`(use "./should-not-be-imported" to import "/Users/user/project/should-not-be-imported.ts")
+/Users/user/project/tsconfig.json: warning: Non-relative path "bad" is not allowed when "baseUrl" is not set (did you forget a leading "./"?)
+/Users/user/project/tsconfig.json: warning: Non-relative path "@bad/core" is not allowed when "baseUrl" is not set (did you forget a leading "./"?)
+/Users/user/project/tsconfig.json: warning: Non-relative path ".*/bad" is not allowed when "baseUrl" is not set (did you forget a leading "./"?)
+/Users/user/project/tsconfig.json: warning: Non-relative path "..*/bad" is not allowed when "baseUrl" is not set (did you forget a leading "./"?)
+/Users/user/project/tsconfig.json: warning: Non-relative path "c*:\\bad" is not allowed when "baseUrl" is not set (did you forget a leading "./"?)
+/Users/user/project/tsconfig.json: warning: Non-relative path "c:*\\bad" is not allowed when "baseUrl" is not set (did you forget a leading "./"?)
+/Users/user/project/tsconfig.json: warning: Non-relative path "http://bad" is not allowed when "baseUrl" is not set (did you forget a leading "./"?)
+`,
+	})
+}
+
 func TestTsConfigJSX(t *testing.T) {
 	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -257,6 +472,34 @@ func TestJsconfigJsonBaseUrl(t *testing.T) {
 	})
 }
 
+func TestTsconfigJsonAbsoluteBaseUrl(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/app/entry.js": `
+				import fn from 'lib/util'
+				console.log(fn())
+			`,
+			"/Users/user/project/src/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"baseUrl": "/Users/user/project/src"
+					}
+				}
+			`,
+			"/Users/user/project/src/lib/util.js": `
+				module.exports = function() {
+					return 123
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/app/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
+
 func TestTsconfigJsonCommentAllowed(t *testing.T) {
 	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -348,32 +591,40 @@ func TestTsconfigJsonExtends(t *testing.T) {
 func TestTsconfigJsonExtendsThreeLevels(t *testing.T) {
 	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
-			"/entry.jsx": `
+			"/Users/user/project/src/entry.jsx": `
+				import "test/import.js"
 				console.log(<div/>, <></>)
 			`,
-			"/tsconfig.json": `
+			"/Users/user/project/src/tsconfig.json": `
 				{
-					"extends": "./base",
+					"extends": "./path1/base",
 					"compilerOptions": {
 						"jsxFragmentFactory": "derivedFragment"
 					}
 				}
 			`,
-			"/base.json": `
+			"/Users/user/project/src/path1/base.json": `
 				{
-					"extends": "./base2"
+					"extends": "../path2/base2"
 				}
 			`,
-			"/base2.json": `
+			"/Users/user/project/src/path2/base2.json": `
 				{
 					"compilerOptions": {
+						"baseUrl": ".",
+						"paths": {
+							"test/*": ["./works/*"]
+						},
 						"jsxFactory": "baseFactory",
 						"jsxFragmentFactory": "baseFragment"
 					}
 				}
 			`,
+			"/Users/user/project/src/path2/works/import.js": `
+				console.log('works')
+			`,
 		},
-		entryPaths: []string{"/entry.jsx"},
+		entryPaths: []string{"/Users/user/project/src/entry.jsx"},
 		options: config.Options{
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
@@ -587,3 +838,50 @@ func TestTsconfigWarningsInsideNodeModules(t *testing.T) {
 `,
 	})
 }
+
+func TestTsconfigRemoveTypeImports(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.ts": `
+				import {x, y} from "./foo"
+				console.log(1 as x)
+			`,
+			"/Users/user/project/src/tsconfig.json": `{
+				"compilerOptions": {
+					"importsNotUsedAsValues": "remove"
+				}
+			}`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
+
+func TestTsconfigPreserveTypeImports(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.ts": `
+				import {x, y} from "./foo"
+				console.log(1 as x)
+			`,
+			"/Users/user/project/src/tsconfig.json": `{
+				"compilerOptions": {
+					"importsNotUsedAsValues": "preserve"
+				}
+			}`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+			ExternalModules: config.ExternalModules{
+				AbsPaths: map[string]bool{
+					"/Users/user/project/src/foo": true,
+				},
+			},
+		},
+	})
+}
