@@ -22,7 +22,7 @@ func expectParseError(t *testing.T, contents string, expected string) {
 	t.Run(contents, func(t *testing.T) {
 		t.Helper()
 		log := logger.NewDeferLog()
-		Parse(log, test.SourceForTest(contents), config.Options{})
+		Parse(log, test.SourceForTest(contents), Options{})
 		msgs := log.Done()
 		text := ""
 		for _, msg := range msgs {
@@ -37,7 +37,10 @@ func expectPrintedCommon(t *testing.T, name string, contents string, expected st
 	t.Run(name, func(t *testing.T) {
 		t.Helper()
 		log := logger.NewDeferLog()
-		tree := Parse(log, test.SourceForTest(contents), options)
+		tree := Parse(log, test.SourceForTest(contents), Options{
+			MangleSyntax:           options.MangleSyntax,
+			UnsupportedCSSFeatures: options.UnsupportedCSSFeatures,
+		})
 		msgs := log.Done()
 		text := ""
 		for _, msg := range msgs {
@@ -70,6 +73,154 @@ func expectPrintedMangle(t *testing.T, contents string, expected string) {
 	})
 }
 
+func TestEscapes(t *testing.T) {
+	// TIdent
+	expectPrinted(t, "a { value: id\\65nt }", "a {\n  value: ident;\n}\n")
+	expectPrinted(t, "a { value: \\69 dent }", "a {\n  value: ident;\n}\n")
+	expectPrinted(t, "a { value: \\69dent }", "a {\n  value: \u69DEnt;\n}\n")
+	expectPrinted(t, "a { value: \\2cx }", "a {\n  value: \\,x;\n}\n")
+	expectPrinted(t, "a { value: \\,x }", "a {\n  value: \\,x;\n}\n")
+	expectPrinted(t, "a { value: x\\2c }", "a {\n  value: x\\,;\n}\n")
+	expectPrinted(t, "a { value: x\\, }", "a {\n  value: x\\,;\n}\n")
+
+	// THash
+	expectPrinted(t, "a { value: #0h\\61sh }", "a {\n  value: #0hash;\n}\n")
+	expectPrinted(t, "a { value: #\\30hash }", "a {\n  value: #0hash;\n}\n")
+	expectPrinted(t, "a { value: #\\2cx }", "a {\n  value: #\\,x;\n}\n")
+	expectPrinted(t, "a { value: #\\,x }", "a {\n  value: #\\,x;\n}\n")
+
+	// THashID
+	expectPrinted(t, "a { value: #h\\61sh }", "a {\n  value: #hash;\n}\n")
+	expectPrinted(t, "a { value: #\\68 ash }", "a {\n  value: #hash;\n}\n")
+	expectPrinted(t, "a { value: #\\68ash }", "a {\n  value: #\u068Ash;\n}\n")
+	expectPrinted(t, "a { value: #x\\2c }", "a {\n  value: #x\\,;\n}\n")
+	expectPrinted(t, "a { value: #x\\, }", "a {\n  value: #x\\,;\n}\n")
+
+	// TFunction
+	expectPrinted(t, "a { value: f\\6e() }", "a {\n  value: fn();\n}\n")
+	expectPrinted(t, "a { value: \\66n() }", "a {\n  value: fn();\n}\n")
+	expectPrinted(t, "a { value: \\2cx() }", "a {\n  value: \\,x();\n}\n")
+	expectPrinted(t, "a { value: \\,x() }", "a {\n  value: \\,x();\n}\n")
+	expectPrinted(t, "a { value: x\\2c() }", "a {\n  value: x\\,();\n}\n")
+	expectPrinted(t, "a { value: x\\,() }", "a {\n  value: x\\,();\n}\n")
+
+	// TString
+	expectPrinted(t, "a { value: 'a\\62 c' }", "a {\n  value: \"abc\";\n}\n")
+	expectPrinted(t, "a { value: 'a\\62c' }", "a {\n  value: \"a\u062C\";\n}\n")
+	expectPrinted(t, "a { value: '\\61 bc' }", "a {\n  value: \"abc\";\n}\n")
+	expectPrinted(t, "a { value: '\\61bc' }", "a {\n  value: \"\u61BC\";\n}\n")
+	expectPrinted(t, "a { value: '\\2c' }", "a {\n  value: \",\";\n}\n")
+	expectPrinted(t, "a { value: '\\,' }", "a {\n  value: \",\";\n}\n")
+
+	// TURL
+	expectPrinted(t, "a { value: url(a\\62 c) }", "a {\n  value: url(abc);\n}\n")
+	expectPrinted(t, "a { value: url(a\\62c) }", "a {\n  value: url(a\u062C);\n}\n")
+	expectPrinted(t, "a { value: url(\\61 bc) }", "a {\n  value: url(abc);\n}\n")
+	expectPrinted(t, "a { value: url(\\61bc) }", "a {\n  value: url(\u61BC);\n}\n")
+	expectPrinted(t, "a { value: url(\\2c) }", "a {\n  value: url(,);\n}\n")
+	expectPrinted(t, "a { value: url(\\,) }", "a {\n  value: url(,);\n}\n")
+
+	// TAtKeyword
+	expectPrinted(t, "a { value: @k\\65yword }", "a {\n  value: @keyword;\n}\n")
+	expectPrinted(t, "a { value: @\\6b eyword }", "a {\n  value: @keyword;\n}\n")
+	expectPrinted(t, "a { value: @\\6beyword }", "a {\n  value: @\u06BEyword;\n}\n")
+	expectPrinted(t, "a { value: @\\2cx }", "a {\n  value: @\\,x;\n}\n")
+	expectPrinted(t, "a { value: @\\,x }", "a {\n  value: @\\,x;\n}\n")
+	expectPrinted(t, "a { value: @x\\2c }", "a {\n  value: @x\\,;\n}\n")
+	expectPrinted(t, "a { value: @x\\, }", "a {\n  value: @x\\,;\n}\n")
+
+	// TDimension
+	expectPrinted(t, "a { value: 10\\65m }", "a {\n  value: 10em;\n}\n")
+	expectPrinted(t, "a { value: 10p\\32x }", "a {\n  value: 10p2x;\n}\n")
+	expectPrinted(t, "a { value: 10e\\32x }", "a {\n  value: 10\\65 2x;\n}\n")
+	expectPrinted(t, "a { value: 10\\32x }", "a {\n  value: 10\\32x;\n}\n")
+	expectPrinted(t, "a { value: 10\\2cx }", "a {\n  value: 10\\,x;\n}\n")
+	expectPrinted(t, "a { value: 10\\,x }", "a {\n  value: 10\\,x;\n}\n")
+	expectPrinted(t, "a { value: 10x\\2c }", "a {\n  value: 10x\\,;\n}\n")
+	expectPrinted(t, "a { value: 10x\\, }", "a {\n  value: 10x\\,;\n}\n")
+
+	// RDeclaration
+	expectPrintedMangle(t, "a { c\\6flor: #f00 }", "a {\n  color: red;\n}\n")
+	expectPrintedMangle(t, "a { \\63olor: #f00 }", "a {\n  color: red;\n}\n")
+	expectPrintedMangle(t, "a { \\2color: #f00 }", "a {\n  \\,olor: #f00;\n}\n")
+	expectPrintedMangle(t, "a { \\,olor: #f00 }", "a {\n  \\,olor: #f00;\n}\n")
+
+	// RUnknownAt
+	expectPrinted(t, "@unknown;", "@unknown;\n")
+	expectPrinted(t, "@u\\6eknown;", "@unknown;\n")
+	expectPrinted(t, "@\\75nknown;", "@unknown;\n")
+	expectPrinted(t, "@u\\2cnknown;", "@u\\,nknown;\n")
+	expectPrinted(t, "@u\\,nknown;", "@u\\,nknown;\n")
+	expectPrinted(t, "@\\2cunknown;", "@\\,unknown;\n")
+	expectPrinted(t, "@\\,unknown;", "@\\,unknown;\n")
+
+	// RAtKeyframes
+	expectPrinted(t, "@k\\65yframes abc { from {} }", "@keyframes abc {\n  from {\n  }\n}\n")
+	expectPrinted(t, "@keyframes \\61 bc { from {} }", "@keyframes abc {\n  from {\n  }\n}\n")
+	expectPrinted(t, "@keyframes a\\62 c { from {} }", "@keyframes abc {\n  from {\n  }\n}\n")
+	expectPrinted(t, "@keyframes abc { \\66rom {} }", "@keyframes abc {\n  from {\n  }\n}\n")
+	expectPrinted(t, "@keyframes a\\2c c { \\66rom {} }", "@keyframes a\\,c {\n  from {\n  }\n}\n")
+	expectPrinted(t, "@keyframes a\\,c { \\66rom {} }", "@keyframes a\\,c {\n  from {\n  }\n}\n")
+
+	// RAtNamespace
+	expectPrinted(t, "@n\\61mespace ns 'path';", "@namespace ns \"path\";\n")
+	expectPrinted(t, "@namespace \\6es 'path';", "@namespace ns \"path\";\n")
+	expectPrinted(t, "@namespace ns 'p\\61th';", "@namespace ns \"path\";\n")
+	expectPrinted(t, "@namespace \\2cs 'p\\61th';", "@namespace \\,s \"path\";\n")
+	expectPrinted(t, "@namespace \\,s 'p\\61th';", "@namespace \\,s \"path\";\n")
+
+	// CompoundSelector
+	expectPrinted(t, "* {}", "* {\n}\n")
+	expectPrinted(t, "*|div {}", "*|div {\n}\n")
+	expectPrinted(t, "\\2a {}", "\\* {\n}\n")
+	expectPrinted(t, "\\2a|div {}", "\\*|div {\n}\n")
+	expectPrinted(t, "\\2d {}", "- {\n}\n")
+	expectPrinted(t, "\\2d- {}", "-- {\n}\n")
+	expectPrinted(t, "-\\2d {}", "-- {\n}\n")
+	expectPrinted(t, "\\2d 123 {}", "\\-123 {\n}\n")
+
+	// SSHash
+	expectPrinted(t, "#h\\61sh {}", "#hash {\n}\n")
+	expectPrinted(t, "#\\2chash {}", "#\\,hash {\n}\n")
+	expectPrinted(t, "#\\,hash {}", "#\\,hash {\n}\n")
+	expectPrinted(t, "#\\2d {}", "#- {\n}\n")
+	expectPrinted(t, "#\\2d- {}", "#-- {\n}\n")
+	expectPrinted(t, "#-\\2d {}", "#-- {\n}\n")
+	expectPrinted(t, "#\\2d 123 {}", "#\\-123 {\n}\n")
+	expectPrinted(t, "#\\61hash {}", "#ahash {\n}\n")
+	expectPrinted(t, "#\\30hash {}", "#\\30hash {\n}\n")
+	expectPrinted(t, "#0\\2chash {}", "#0\\,hash {\n}\n")
+	expectPrinted(t, "#0\\,hash {}", "#0\\,hash {\n}\n")
+
+	// SSClass
+	expectPrinted(t, ".cl\\61ss {}", ".class {\n}\n")
+	expectPrinted(t, ".\\2c class {}", ".\\,class {\n}\n")
+	expectPrinted(t, ".\\,class {}", ".\\,class {\n}\n")
+
+	// SSPseudoClass
+	expectPrinted(t, ":pseudocl\\61ss {}", ":pseudoclass {\n}\n")
+	expectPrinted(t, ":pseudo\\2c class {}", ":pseudo\\,class {\n}\n")
+	expectPrinted(t, ":pseudo\\,class {}", ":pseudo\\,class {\n}\n")
+	expectPrinted(t, ":pseudo(cl\\61ss) {}", ":pseudo(class) {\n}\n")
+	expectPrinted(t, ":pseudo(cl\\2css) {}", ":pseudo(cl\\,ss) {\n}\n")
+	expectPrinted(t, ":pseudo(cl\\,ss) {}", ":pseudo(cl\\,ss) {\n}\n")
+
+	// SSAttribute
+	expectPrinted(t, "[\\61ttr] {}", "[attr] {\n}\n")
+	expectPrinted(t, "[\\2c attr] {}", "[\\,attr] {\n}\n")
+	expectPrinted(t, "[\\,attr] {}", "[\\,attr] {\n}\n")
+	expectPrinted(t, "[attr\\7e=x] {}", "[attr\\~=x] {\n}\n")
+	expectPrinted(t, "[attr\\~=x] {}", "[attr\\~=x] {\n}\n")
+	expectPrinted(t, "[attr=\\2c] {}", "[attr=\",\"] {\n}\n")
+	expectPrinted(t, "[attr=\\,] {}", "[attr=\",\"] {\n}\n")
+	expectPrinted(t, "[\\6es|attr] {}", "[ns|attr] {\n}\n")
+	expectPrinted(t, "[ns|\\61ttr] {}", "[ns|attr] {\n}\n")
+	expectPrinted(t, "[\\2cns|attr] {}", "[\\,ns|attr] {\n}\n")
+	expectPrinted(t, "[ns|\\2c attr] {}", "[ns|\\,attr] {\n}\n")
+	expectPrinted(t, "[*|attr] {}", "[*|attr] {\n}\n")
+	expectPrinted(t, "[\\2a|attr] {}", "[\\*|attr] {\n}\n")
+}
+
 func TestString(t *testing.T) {
 	expectPrinted(t, "a:after { content: 'a\\\rb' }", "a:after {\n  content: \"ab\";\n}\n")
 	expectPrinted(t, "a:after { content: 'a\\\nb' }", "a:after {\n  content: \"ab\";\n}\n")
@@ -337,6 +488,14 @@ func TestDeclaration(t *testing.T) {
 	expectPrinted(t, ".decl { a: b; c: d; }", ".decl {\n  a: b;\n  c: d;\n}\n")
 	expectParseError(t, ".decl { a { b: c; } }", "<stdin>: warning: Expected \":\" but found \"{\"\n")
 	expectPrinted(t, ".decl { & a { b: c; } }", ".decl {\n  & a {\n    b: c;\n  }\n}\n")
+
+	// See http://browserhacks.com/
+	expectPrinted(t, ".selector { (;property: value;); }", ".selector {\n  (;property: value;);\n}\n")
+	expectPrinted(t, ".selector { [;property: value;]; }", ".selector {\n  [;property: value;];\n}\n")
+	expectPrinted(t, ".selector, {}", ".selector, {\n}\n")
+	expectPrinted(t, ".selector\\ {}", ".selector\\  {\n}\n")
+	expectPrinted(t, ".selector { property: value\\9; }", ".selector {\n  property: value\\\t;\n}\n")
+	expectPrinted(t, "@media \\0screen\\,screen\\9 {}", "@media \uFFFDscreen\\,screen\\\t {\n}\n")
 }
 
 func TestSelector(t *testing.T) {
@@ -344,6 +503,14 @@ func TestSelector(t *testing.T) {
 	expectPrinted(t, "a {}", "a {\n}\n")
 	expectPrinted(t, "a b {}", "a b {\n}\n")
 
+	expectPrinted(t, "a/**/b {}", "a b {\n}\n")
+	expectPrinted(t, "a/**/.b {}", "a.b {\n}\n")
+	expectPrinted(t, "a/**/:b {}", "a:b {\n}\n")
+	expectPrinted(t, "a/**/[b] {}", "a[b] {\n}\n")
+	expectPrinted(t, "a>/**/b {}", "a > b {\n}\n")
+	expectPrinted(t, "a+/**/b {}", "a + b {\n}\n")
+	expectPrinted(t, "a~/**/b {}", "a ~ b {\n}\n")
+
 	expectPrinted(t, "[b]{}", "[b] {\n}\n")
 	expectPrinted(t, "[b] {}", "[b] {\n}\n")
 	expectPrinted(t, "a[b] {}", "a[b] {\n}\n")
@@ -357,30 +524,32 @@ func TestSelector(t *testing.T) {
 	expectPrinted(t, "[|b]{}", "[b] {\n}\n") // "[|b]" is equivalent to "[b]"
 	expectPrinted(t, "[*|b]{}", "[*|b] {\n}\n")
 	expectPrinted(t, "[a|b]{}", "[a|b] {\n}\n")
-	expectPrinted(t, "[a|b|=\"c\"]{}", "[a|b|=\"c\"] {\n}\n")
-	expectPrinted(t, "[a|b |= \"c\"]{}", "[a|b|=\"c\"] {\n}\n")
+	expectPrinted(t, "[a|b|=\"c\"]{}", "[a|b|=c] {\n}\n")
+	expectPrinted(t, "[a|b |= \"c\"]{}", "[a|b|=c] {\n}\n")
 	expectParseError(t, "[a||b] {}", "<stdin>: warning: Expected identifier but found \"|\"\n")
 	expectParseError(t, "[* | b] {}", "<stdin>: warning: Expected \"|\" but found whitespace\n")
 	expectParseError(t, "[a | b] {}", "<stdin>: warning: Expected \"=\" but found whitespace\n")
 
-	expectPrinted(t, "[b=\"c\"] {}", "[b=\"c\"] {\n}\n")
-	expectPrinted(t, "[b~=\"c\"] {}", "[b~=\"c\"] {\n}\n")
-	expectPrinted(t, "[b^=\"c\"] {}", "[b^=\"c\"] {\n}\n")
-	expectPrinted(t, "[b$=\"c\"] {}", "[b$=\"c\"] {\n}\n")
-	expectPrinted(t, "[b*=\"c\"] {}", "[b*=\"c\"] {\n}\n")
-	expectPrinted(t, "[b|=\"c\"] {}", "[b|=\"c\"] {\n}\n")
+	expectPrinted(t, "[b=\"c\"] {}", "[b=c] {\n}\n")
+	expectPrinted(t, "[b=\"c d\"] {}", "[b=\"c d\"] {\n}\n")
+	expectPrinted(t, "[b=\"0c\"] {}", "[b=\"0c\"] {\n}\n")
+	expectPrinted(t, "[b~=\"c\"] {}", "[b~=c] {\n}\n")
+	expectPrinted(t, "[b^=\"c\"] {}", "[b^=c] {\n}\n")
+	expectPrinted(t, "[b$=\"c\"] {}", "[b$=c] {\n}\n")
+	expectPrinted(t, "[b*=\"c\"] {}", "[b*=c] {\n}\n")
+	expectPrinted(t, "[b|=\"c\"] {}", "[b|=c] {\n}\n")
 	expectParseError(t, "[b?=\"c\"] {}", "<stdin>: warning: Expected \"]\" but found \"?\"\n")
 
-	expectPrinted(t, "[b = \"c\"] {}", "[b=\"c\"] {\n}\n")
-	expectPrinted(t, "[b ~= \"c\"] {}", "[b~=\"c\"] {\n}\n")
-	expectPrinted(t, "[b ^= \"c\"] {}", "[b^=\"c\"] {\n}\n")
-	expectPrinted(t, "[b $= \"c\"] {}", "[b$=\"c\"] {\n}\n")
-	expectPrinted(t, "[b *= \"c\"] {}", "[b*=\"c\"] {\n}\n")
-	expectPrinted(t, "[b |= \"c\"] {}", "[b|=\"c\"] {\n}\n")
+	expectPrinted(t, "[b = \"c\"] {}", "[b=c] {\n}\n")
+	expectPrinted(t, "[b ~= \"c\"] {}", "[b~=c] {\n}\n")
+	expectPrinted(t, "[b ^= \"c\"] {}", "[b^=c] {\n}\n")
+	expectPrinted(t, "[b $= \"c\"] {}", "[b$=c] {\n}\n")
+	expectPrinted(t, "[b *= \"c\"] {}", "[b*=c] {\n}\n")
+	expectPrinted(t, "[b |= \"c\"] {}", "[b|=c] {\n}\n")
 	expectParseError(t, "[b ?= \"c\"] {}", "<stdin>: warning: Expected \"]\" but found \"?\"\n")
 
-	expectPrinted(t, "[b = \"c\" i] {}", "[b=\"c\" i] {\n}\n")
-	expectPrinted(t, "[b = \"c\" I] {}", "[b=\"c\" I] {\n}\n")
+	expectPrinted(t, "[b = \"c\" i] {}", "[b=c i] {\n}\n")
+	expectPrinted(t, "[b = \"c\" I] {}", "[b=c I] {\n}\n")
 	expectParseError(t, "[b i] {}", "<stdin>: warning: Expected \"]\" but found \"i\"\n<stdin>: warning: Unexpected \"]\"\n")
 	expectParseError(t, "[b I] {}", "<stdin>: warning: Expected \"]\" but found \"I\"\n<stdin>: warning: Unexpected \"]\"\n")
 
@@ -410,6 +579,20 @@ func TestSelector(t *testing.T) {
 	expectPrinted(t, "a::b(c) {}", "a::b(c) {\n}\n")
 	expectPrinted(t, "a:b:c {}", "a:b:c {\n}\n")
 	expectPrinted(t, "a:b(:c) {}", "a:b(:c) {\n}\n")
+	expectPrinted(t, "a: b {}", "a: b {\n}\n")
+
+	expectPrinted(t, "#id {}", "#id {\n}\n")
+	expectPrinted(t, "#--0 {}", "#--0 {\n}\n")
+	expectPrinted(t, "#\\-0 {}", "#\\-0 {\n}\n")
+	expectPrinted(t, "#\\30 {}", "#\\30  {\n}\n")
+	expectPrinted(t, "div#id {}", "div#id {\n}\n")
+	expectPrinted(t, "div#--0 {}", "div#--0 {\n}\n")
+	expectPrinted(t, "div#\\-0 {}", "div#\\-0 {\n}\n")
+	expectPrinted(t, "div#\\30 {}", "div#\\30  {\n}\n")
+	expectParseError(t, "#0 {}", "<stdin>: warning: Unexpected \"#0\"\n")
+	expectParseError(t, "#-0 {}", "<stdin>: warning: Unexpected \"#-0\"\n")
+	expectParseError(t, "div#0 {}", "<stdin>: warning: Unexpected \"#0\"\n")
+	expectParseError(t, "div#-0 {}", "<stdin>: warning: Unexpected \"#-0\"\n")
 }
 
 func TestNestedSelector(t *testing.T) {
@@ -536,6 +719,7 @@ func TestAtImport(t *testing.T) {
 }
 
 func TestAtKeyframes(t *testing.T) {
+	expectPrinted(t, "@keyframes {}", "@keyframes \"\" {\n}\n")
 	expectPrinted(t, "@keyframes name{}", "@keyframes name {\n}\n")
 	expectPrinted(t, "@keyframes name {}", "@keyframes name {\n}\n")
 	expectPrinted(t, "@keyframes name{0%,50%{color:red}25%,75%{color:blue}}",
